from .dan_fever import DanFever
from .swefaq_retrieval import SweFaqRetrieval
from .swedn_retrieval import SwednRetrieval
from .swefaq_retrieval import SweFaqRetrieval
from .snl_retrieval import SNLRetrieval
from .t2nord_retrieval import TV2Nordretrieval
from .norquad import NorQuadRetrieval
from .twitterhjerne import TwitterHjerneRetrieval